// Digital Radio Controller
// 2014.3.18 Eleken

// main clock : 128k Hz
// FUSE BITS HIGH: 11011101
// FUSE BITS LOW : 11010110

#include "main.h"

struct sRadioSettings g_radioSettings;

/* EEPROM */
uint8_t eep_dummy __attribute__((section(".eeprom")));
uint8_t eep_area[sizeof(g_radioSettings)] __attribute__((section(".eeprom")));

static void LoadEEP(void){ // EEPROM -> RAM
	uint8_t i;
	uint8_t* p = (uint8_t*)(&g_radioSettings);
	for(i=0; i<sizeof(g_radioSettings); i++){
		uint8_t a = eeprom_read_byte(&(eep_area[i]));
		if(a != 0xFF) p[i] = a;
	}
}
static void SaveEEP(void){ // RAM -> EEPROM 
	uint8_t i;
	uint8_t* p = (uint8_t*)(&g_radioSettings);
	for(i=0; i<sizeof(g_radioSettings); i++){
		eeprom_write_byte(&(eep_area[i]), p[i]);
	}
}

uint8_t CheckSW(void){
	static uint8_t sw_old = SW_PORT_MASK;
	uint8_t sw = (SW_PORT & SW_PORT_MASK);
	uint8_t ret = SW_UNPUSHED;
	
	if((sw_old == SW1_PUSHED) && (sw == SW_PORT_MASK)){ // sw1 released
		ret = SW1;
	}
	
	sw_old = sw;
	return ret;
}

static void ChangeChannel(void){
	switch(g_radioSettings.channel){
	case CHANNEL_FM:
		DDRD = 0b01100000;
		PORTB = (PORTB & 0b10000000) | 0b01110001; // F
		break;
	case CHANNEL_MW:
		DDRD = 0b01010000; // 1111
		PORTB = (PORTB & 0b10000000) | 0b01110111; // A
		break;
	case CHANNEL_SW1:
		DDRD = 0b01010101; // 1010
		PORTB = (PORTB & 0b10000000) | 0b00000110; // 1
		break;
	case CHANNEL_SW2:
		DDRD = 0b01011101; // 0010
		PORTB = (PORTB & 0b10000000) | 0b01011011; // 2
		break;
	default:
		DDRD = 0b01011111; // 0000
		PORTB = (PORTB & 0b10000000) | 0b01001111; // 3
		break;
	}
}
static void InitPort(void){
	ACSR  = 0b10000000;

	DDRD  = 0b01100000;
	PORTD = 0b00000000;
	
	DDRB  = 0b01111111;
	PORTB = 0b10000000;
	
	DDRA  = 0;
	PORTA = 0b00000010;
}

int main(void){
	// Load initial value (Radio)
	g_radioSettings.channel = CHANNEL_FM; //FM

	InitPort();
	// if SW1 is pushed, skip loading EEPROM data
	if((SW_PORT & SW_PORT_MASK) != SW1_PUSHED) LoadEEP();
	ChangeChannel();
	
	while(1){
		// check switch
		uint8_t a = CheckSW();
		if(a == SW1){
			switch(g_radioSettings.channel){
			case CHANNEL_FM:
				g_radioSettings.channel = CHANNEL_MW;
				break;
			case CHANNEL_MW:
				g_radioSettings.channel = CHANNEL_SW1;
				break;
			case CHANNEL_SW1:
				g_radioSettings.channel = CHANNEL_SW2;
				break;
			case CHANNEL_SW2:
				g_radioSettings.channel = CHANNEL_SW3;
				break;
			default:
				g_radioSettings.channel = CHANNEL_FM;
				break;
			}
			ChangeChannel();
			SaveEEP();
		}
		
		if(PINA & 0x01) SBI(PORTD, PD6);
		else CBI(PORTD, PD6);
		_delay_ms(100);
	}

	return 0;
}
